#include <cv.h>
#include <stdio.h>
#include <stdlib.h>

#define cvM32f(Matrix,i,j) CV_MAT_ELEM(*(Matrix),float,(i),(j))
#define cvV32f(Matrix,i) Matrix->data.fl[i]

//Error Handler Function
void DieWithError(char*ErrorMsg){
	perror(ErrorMsg);	//Display Error Message
	exit(1);		//End Application
}




void cvModernPOSIT(CvMat*x,CvMat*y,CvMat*WorldPoints,float FocalLength,CvPoint ImageCenter,CvMat*R,CvMat*T){
	int nPoints,i;
	CvMat*CenteredImage=NULL,*ui=NULL,*vi=NULL,*wi=NULL,*HomogeneousWorldPoints=NULL,*ObjectMat=NULL;

	//Test Input Data
	if(x==NULL||y==NULL||WorldPoints==NULL||FocalLength<0||R==NULL||T==NULL) DieWithError("ModernPOSIT: Invalid Inputs");

	//Get Number of Points
	nPoints=WorldPoints->rows;
	
	//Alloc Centered Image Data
	CenteredImage=cvCreateMat(nPoints,2,CV_32FC1);
	

	for(i=0;i<nPoints;i++){ cvM32f(CenteredImage,i,0)=(cvV32f(x,i)-ImageCenter.x)/FocalLength;	
				cvM32f(CenteredImage,i,1)=(cvV32f(y,i)-ImageCenter.y)/FocalLength;
	}


	ui=cvCreateMat(nPoints,1,CV_32FC1);
	vi=cvCreateMat(nPoints,1,CV_32FC1);
	wi=cvCreateMat(nPoints,1,CV_32FC1);

	HomogeneousWorldPoints=cvCreateMat(nPoints,4,CV_32FC1);
	ObjectMat=cvCreateMat(4,nPoints,CV_32FC1);


	for(i=0;i<nPoints;i++){	cvM32f(ui,i,0)=cvM32f(CenteredImage,i,0);
				cvM32f(vi,i,0)=cvM32f(CenteredImage,i,1);
				cvM32f(wi,i,0)=1;
				cvM32f(HomogeneousWorldPoints,i,0)=cvM32f(WorldPoints,i,0);
				cvM32f(HomogeneousWorldPoints,i,1)=cvM32f(WorldPoints,i,1);
				cvM32f(HomogeneousWorldPoints,i,2)=cvM32f(WorldPoints,i,2);
				cvM32f(HomogeneousWorldPoints,i,3)=1;
	}
	

	cvInvert(HomogeneousWorldPoints,ObjectMat, CV_SVD);
	
	//cvLoadMat("objectMat.txt",ObjectMat); 




	CvMat*r1T=cvCreateMat(4,1,CV_32FC1);
	CvMat*r2T=cvCreateMat(4,1,CV_32FC1);
	float Tz1,Tz2,Tz,Tx,Ty,Delta;
	CvMat*r1N=cvCreateMat(4,1,CV_32FC1);
	CvMat*r2N=cvCreateMat(4,1,CV_32FC1);
	CvMat*r1=cvCreateMat(3,1,CV_32FC1);
	CvMat*r2=cvCreateMat(3,1,CV_32FC1);
	CvMat*r3=cvCreateMat(3,1,CV_32FC1);
	CvMat*r3T=cvCreateMat(4,1,CV_32FC1);

	CvMat*oldUi=cvCreateMat(nPoints,1,CV_32FC1);
	CvMat*oldVi=cvCreateMat(nPoints,1,CV_32FC1);
	CvMat*DeltaUi=cvCreateMat(nPoints,1,CV_32FC1);
	CvMat*DeltaVi=cvCreateMat(nPoints,1,CV_32FC1);


	int Converged=0,Count=0;


while(!Converged){

	//r1T=ObjectMat*ui
	cvGEMM(ObjectMat,ui,1.0,NULL,0,r1T,0);

	//r2T=ObjectMat*vi
	cvGEMM(ObjectMat,vi,1.0,NULL,0,r2T,0);

	//Tz1 = 1/sqrt(r1T(1)*r1T(1) + r1T(2)*r1T(2)+ r1T(3)*r1T(3))
	Tz1=1/sqrt(cvM32f(r1T,0,0)*cvM32f(r1T,0,0) + cvM32f(r1T,1,0)*cvM32f(r1T,1,0) + cvM32f(r1T,2,0)*cvM32f(r1T,2,0));

	//Tz2 = 1/sqrt(r2T(1)*r2T(1) + r2T(2)*r2T(2)+ r2T(3)*r2T(3))
	Tz2=1/sqrt(cvM32f(r2T,0,0)*cvM32f(r2T,0,0) + cvM32f(r2T,1,0)*cvM32f(r2T,1,0) + cvM32f(r2T,2,0)*cvM32f(r2T,2,0));

	Tz = sqrt(Tz1*Tz2);

	//r1N = r1T*Tz;
	//r2N = r2T*Tz;
	for(i=0;i<4;i++){ 	cvM32f(r1N,i,0)=cvM32f(r1T,i,0)*Tz;
				cvM32f(r2N,i,0)=cvM32f(r2T,i,0)*Tz;
	}

	//r1 = r1N(1:3);
	//r2 = r2N(1:3);
	for(i=0;i<3;i++){ 	cvM32f(r1,i,0)=cvM32f(r1N,i,0);
				cvM32f(r2,i,0)=cvM32f(r2N,i,0);
	}

	//r3 = cross(r1,r2);
	cvCrossProduct(r1,r2,r3);

	
	//r3T= [r3; Tz]
	for(i=0;i<3;i++)	cvM32f(r3T,i,0)=cvM32f(r3,i,0);
	cvM32f(r3T,3,0)=Tz;


	Tx=cvM32f(r1N,3,0);
	Ty=cvM32f(r2N,3,0);
	
	//wi= homogeneousWorldPts * r3T /Tz;
	cvGEMM(HomogeneousWorldPoints,r3T,1.0/Tz,NULL,0,wi,0);

	//oldUi = ui;
	//oldVi = vi;
	for(i=0;i<nPoints;i++){	cvM32f(oldUi,i,0)=cvM32f(ui,i,0);
				cvM32f(oldVi,i,0)=cvM32f(vi,i,0);
	}

	//ui = wi .* centeredImage(:,1);
	//vi = wi .* centeredImage(:,2);
	for(i=0;i<nPoints;i++){	cvM32f(ui,i,0)=cvM32f(wi,i,0)*cvM32f(CenteredImage,i,0);
				cvM32f(vi,i,0)=cvM32f(wi,i,0)*cvM32f(CenteredImage,i,1);
	}

	//deltaUi = ui - oldUi;
	//deltaVi = vi - oldVi;
	for(i=0;i<nPoints;i++){	cvM32f(DeltaUi,i,0)=cvM32f(ui,i,0)-cvM32f(oldUi,i,0);
				cvM32f(DeltaVi,i,0)=cvM32f(vi,i,0)-cvM32f(oldVi,i,0);
	}

	//delta = focalLength * focalLength * (deltaUi' * deltaUi + deltaVi' * deltaVi)
	Delta=0;
	for(i=0;i<nPoints;i++) Delta+=cvM32f(DeltaUi,i,0)*cvM32f(DeltaUi,i,0)+cvM32f(DeltaVi,i,0)*cvM32f(DeltaVi,i,0);
	Delta*=FocalLength*FocalLength;	

	//printf("Delta=%g\n",Delta);

	Converged=(Count>0 & Delta<1);
	Count++;
}


	//trans = [Tx; Ty; Tz];
	cvM32f(T,0,0)=Tx; cvM32f(T,1,0)=Ty; cvM32f(T,2,0)=Tz;

	//rot = [r1'; r2'; r3'];
	for(i=0;i<3;i++){ 
			cvM32f(R,0,i)=cvM32f(r1,i,0);
			cvM32f(R,1,i)=cvM32f(r2,i,0);
			cvM32f(R,2,i)=cvM32f(r3,i,0);
	}


	cvReleaseMat(&r1T);
	cvReleaseMat(&r2T);
	cvReleaseMat(&r1N);
	cvReleaseMat(&r2N);
	cvReleaseMat(&r1);
	cvReleaseMat(&r2);
	cvReleaseMat(&r3);
	cvReleaseMat(&r3T);
	cvReleaseMat(&oldUi);
	cvReleaseMat(&oldVi);

	//Release Data
	cvReleaseMat(&CenteredImage);
	cvReleaseMat(&ui);
	cvReleaseMat(&vi);
	cvReleaseMat(&wi);
	cvReleaseMat(&HomogeneousWorldPoints);
	cvReleaseMat(&ObjectMat);
}

 
